"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Linting = exports.Linter = void 0;
const ErrorMessages_1 = require("./ErrorMessages");
const Logger_1 = require("./Logger");
function snippetExampleAsComment(codeExample) {
    let codeExampleSnippet = codeExample.content.split("\n");
    codeExample.exampleText.split("\n")
        .forEach((line, i) => {
        codeExampleSnippet.unshift(Logger_1.colors.Wrap(Logger_1.colors.Dim, Logger_1.colors.WrapUnderscore('// ' + line)));
    });
    let final = codeExampleSnippet.join('\n');
    if (codeExample.includeCodeSnippetLocation === 'before') {
        final = '\n' + final;
    }
    else {
        final = final + '\n';
    }
    return final;
}
function printCodeLines(code, startLineNumber = 0, maxLines = 10, codeExample = null) {
    const snippetLines = code.split("\n");
    let i = 0;
    if (codeExample && (codeExample === null || codeExample === void 0 ? void 0 : codeExample.includeCodeSnippet) && (codeExample === null || codeExample === void 0 ? void 0 : codeExample.includeCodeSnippetLocation) === 'after') {
        i = printCodeLines(snippetExampleAsComment(codeExample), i);
        maxLines += i;
    }
    for (i; i < snippetLines.length; i++) {
        const line = snippetLines[i];
        const lineNum = (i + startLineNumber + 1).toString().padStart(3, ' ');
        if (i > maxLines) {
            let j = 0;
            for (j; j < 3; j++) {
                const cutoffNum = (i + j + startLineNumber + 1).toString().padStart(3, ' ');
                process.stdout.write(Logger_1.colors.Dim + cutoffNum + Logger_1.colors.Reset + ' | ' + Logger_1.colors.Dim + '.'.repeat(3 - j) + Logger_1.colors.Reset);
                process.stdout.write("\n");
            }
            i = j;
            break;
        }
        process.stdout.write(Logger_1.colors.Dim + lineNum + Logger_1.colors.Reset + ' | ' + line);
        process.stdout.write("\n");
    }
    if (codeExample && (codeExample === null || codeExample === void 0 ? void 0 : codeExample.includeCodeSnippet) && (codeExample === null || codeExample === void 0 ? void 0 : codeExample.includeCodeSnippetLocation) === 'before') {
        printCodeLines(snippetExampleAsComment(codeExample), (snippetLines.length) + 2);
    }
    return i;
}
class Linter {
    constructor(isMainInstance = true) {
        Object.defineProperty(this, "isMainInstance", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: isMainInstance
        });
        Object.defineProperty(this, "_sourceFile", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_messages", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
    }
    factory(sourceFile) {
        const inst = new Linter(false);
        inst._sourceFile = sourceFile;
        return inst;
    }
    merge() {
        Linter.messages = Linter.messages.concat(this._messages);
    }
    //	public info(messages: string, node: ts.Node): void;
    //	public info(messages: string[], node: ts.Node): void;
    info(messages, node) {
        this._add(messages, node, 'info');
    }
    //	public warn(messages: string, node: ts.Node): void;
    //	public warn(messages: string[], node: ts.Node): void;
    warn(messages, node) {
        this._add(messages, node, 'warning');
    }
    //	public error(messages: string, node: ts.Node): void;
    //	public error(messages: string[], node: ts.Node): void;
    error(messages, node) {
        this._add(messages, node, 'error');
    }
    _add(messages, node, type) {
        if (typeof messages === 'string') {
            messages = [messages];
        }
        const message = {
            type: type,
            messages: messages,
            start: node.getStart(this._sourceFile),
            end: node.getEnd(),
            codeSnippet: null,
        };
        if (message.start !== -1) {
            const nodeText = this._sourceFile.text.slice(message.start, message.end);
            const filePos = this._sourceFile.getLineAndCharacterOfPosition(message.start);
            message.codeSnippet = {
                startLine: filePos.line,
                code: nodeText.trim(),
                location: `${this._sourceFile.fileName}:${filePos.line + 1}:${filePos.character}`,
            };
        }
        this._messages.push(message);
    }
    reset() {
        Linter.messages = [];
    }
    print() {
        if (!this.isMainInstance) {
            throw new Error('Cannot print from a non-main instance');
        }
        if (Linter.messages.length === 0) {
            return;
        }
        const messages = Linter.messages;
        Linter.messages = [];
        const [width, height] = process.stdout.getWindowSize();
        for (let message of messages) {
            // Write our message type banner
            const bannerText = '[' + message.type.toUpperCase() + ']';
            const bannerWidth = width - bannerText.length;
            const banner = Logger_1.colors.WrapBgForLog(message.type, bannerText + (' '.repeat(bannerWidth)));
            process.stdout.write(banner);
            process.stdout.write("\n");
            let messageExampleSnippet = null;
            let messageLines = [];
            for (let messageLine of message.messages) {
                if (messageLine instanceof ErrorMessages_1.CodeSnippet) {
                    messageExampleSnippet = messageLine;
                    continue;
                }
                messageLines.push(' ' + messageLine);
            }
            process.stdout.write(messageLines.join('\n'));
            process.stdout.write("\n");
            if (messageExampleSnippet && !(messageExampleSnippet === null || messageExampleSnippet === void 0 ? void 0 : messageExampleSnippet.includeCodeSnippet)) {
                process.stdout.write("\n");
                process.stdout.write(Logger_1.colors.Underscore + messageExampleSnippet.exampleText + Logger_1.colors.Reset);
                process.stdout.write("\n");
                printCodeLines('\n' + messageExampleSnippet.content + '\n');
                process.stdout.write("\n");
            }
            if (message.codeSnippet) {
                process.stdout.write("\n");
                process.stdout.write(Logger_1.colors.Underscore + "Location:" + Logger_1.colors.Reset);
                process.stdout.write("\n");
                process.stdout.write(Logger_1.colors.Dim + message.codeSnippet.location + Logger_1.colors.Reset);
                process.stdout.write("\n");
                process.stdout.write("\n");
                printCodeLines(message.codeSnippet.code, message.codeSnippet.startLine, 10, (messageExampleSnippet === null || messageExampleSnippet === void 0 ? void 0 : messageExampleSnippet.canIncludeCodeSnippet) ? messageExampleSnippet : null);
                process.stdout.write("\n");
                //				if (messageExampleSnippet?.includeCodeSnippet && messageExampleSnippet?.includeCodeSnippetLocation === 'before') {
                //					process.stdout.write("\n");
                //					process.stdout.write(colors.Underscore + messageExampleSnippet.exampleText + colors.Reset);
                //					process.stdout.write("\n");
                //					printCodeLines('\n' + messageExampleSnippet.content + '\n');
                //					process.stdout.write("\n");
                //				}
            }
        }
    }
}
exports.Linter = Linter;
Object.defineProperty(Linter, "messages", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: []
});
exports.Linting = new Linter();
//# sourceMappingURL=Linting.js.map