"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginConfig = void 0;
const fs_jetpack_1 = __importDefault(require("fs-jetpack"));
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const Timed_1 = require("./Utils/Timed");
const ViteFunction_1 = require("./Utils/ViteFunction");
class PluginConfigInstance {
    constructor() {
        /**
         * The project root dir
         * @type {string}
         */
        Object.defineProperty(this, "projectRoot", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * Filesystem instance for the project root
         * @type {typeof jetpack}
         */
        Object.defineProperty(this, "projectRootDirectory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The path to where our stores are located
         * @type {string}
         */
        Object.defineProperty(this, "storesPath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * Filesystem instance for the stores path
         * @type {typeof jetpack}
         */
        Object.defineProperty(this, "storesDirectory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The name of the vue declaration file that will be generated
         * For ex: stores.d.ts
         * @type {string}
         */
        Object.defineProperty(this, "storesFileName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 'stores.d.ts'
        });
        /**
         * The name of the store loader file that will be generated
         * This contains an array of all the stores and their meta.
         * @type {string}
         */
        Object.defineProperty(this, "storeLoaderFile", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 'StoreLoader.ts'
        });
        /**
         * The directory name inside the stores path where
         * all generated files will be written
         * @type {string}
         */
        Object.defineProperty(this, "generatedDirName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 'Generated'
        });
        /**
         * Filesystem instance for the generated files path
         * @type {typeof jetpack}
         */
        Object.defineProperty(this, "generatedDir", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The resolved vite configuration
         *
         * @type {ResolvedConfig}
         * @private
         */
        Object.defineProperty(this, "viteConfig", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_tsConfigPath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_parsedCommandLine", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The file paths to all of our store files
         *
         * @type {string[]}
         */
        Object.defineProperty(this, "storeFilePaths", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
    }
    setUserConfig(config) {
        if (config.projectRoot) {
            this.projectRoot = config.projectRoot;
        }
        if (config.storesPath) {
            this.storesPath = config.storesPath;
        }
        if (config.storesFileName) {
            this.storesFileName = config.storesFileName;
        }
    }
    init(config) {
        if (!this.projectRoot) {
            this.projectRoot = config.root;
        }
        this.storesPath = path_1.default.resolve(this.projectRoot, this.storesPath);
        this.storesDirectory = fs_jetpack_1.default.cwd(this.storesPath);
        this.generatedDir = this.storesDirectory.cwd(this.generatedDirName);
        this.projectRootDirectory = fs_jetpack_1.default.cwd(this.projectRoot);
        this.viteConfig = config;
        this.setStoreFilePaths();
    }
    get tsConfigPath() {
        if (this._tsConfigPath) {
            return this._tsConfigPath;
        }
        const configPath = typescript_1.default.findConfigFile(this.projectRoot, typescript_1.default.sys.fileExists, "tsconfig.json");
        if (!configPath) {
            throw new Error("Could not find a valid 'tsconfig.json'.");
        }
        return this._tsConfigPath = configPath;
    }
    get tsConfig() {
        if (this._parsedCommandLine) {
            return this._parsedCommandLine;
        }
        return this._parsedCommandLine = typescript_1.default.parseJsonConfigFileContent(typescript_1.default.readConfigFile(this.tsConfigPath, typescript_1.default.sys.readFile).config, typescript_1.default.sys, "./");
    }
    get vueDtsPath() {
        return this.generatedDir.path(this.storesFileName);
    }
    get storeLoaderPath() {
        return this.generatedDir.path(this.storeLoaderFile);
    }
    setStoreFilePaths() {
        (0, Timed_1.startTimer)('setStoreFilePaths');
        this.storeFilePaths = this.storesDirectory.find({
            directories: false,
            files: true,
            recursive: true,
            matching: ['*Store.ts', '*store.ts'],
        }).map(filePath => typescript_1.default.sys.resolvePath(this.storesDirectory.path(filePath)));
        (0, Timed_1.endTimer)('setStoreFilePaths');
    }
    /**
     * Checks if the specified file is one that's generated by the plugin.
     * This is used within things like the HMR handler to prevent infinite loops.
     * @param {string} file
     */
    isGeneratedFile(file) {
        const paths = [
            this.vueDtsPath,
            this.storeLoaderPath,
        ];
        return paths.includes(file);
    }
    getRelativeLoaderImportPath(from) {
        return (0, ViteFunction_1.relativeify)(path_1.default.relative(path_1.default.dirname(from), exports.PluginConfig.storesDirectory.path(exports.PluginConfig.generatedDirName, exports.PluginConfig.storeLoaderFile)));
    }
}
exports.PluginConfig = new PluginConfigInstance();
//# sourceMappingURL=PluginConfig.js.map